@@ -3,18 +3,47 @@ import logging
 import pathlib
 import re
 import zipfile
-from collections import namedtuple
+from collections import namedtuple, defaultdict
 from datetime import datetime
+from hashlib import md5
 from io import BytesIO, StringIO
 
-import rebotics_sdk
+import more_itertools
 import tqdm
 
+import rebotics_sdk
+
 logger = logging.getLogger(__name__)
 
-ClassificationEntry = namedtuple('ClassificationEntry', ['label', 'feature', 'image', 'filename'])
+ClassificationEntry = namedtuple('ClassificationEntry', ['label', 'feature', 'image', 'filename', 'index'])
 ImageEntry = namedtuple('ImageEntry', ['filename', 'filepath', 'order'])
 
+
+class VirtualClassificationEntry(object):
+    def __init__(self, label, feature, image_url, index=None):
+        self.label = label
+        self.feature = feature
+        self.image_url = image_url
+        self.index = index
+
+    def to_dict(self):
+        return {
+            'label': self.label,
+            'feature': self.feature,
+            'image_url': self.image_url,
+            'index': self.index
+        }
+
+    @classmethod
+    def from_dict(cls, d):
+        return cls(
+            label=d['label'],
+            feature=d['feature'],
+            image_url=d['image_url'],
+            index=d['index']
+        )
+
+
 NUMERIC_VALUE = re.compile(r'\d+')
 
 
@@ -22,6 +51,12 @@ class ClassificationDatabaseException(Exception):
     pass
 
 
+class DuplicateFeatureVectorsException(ClassificationDatabaseException):
+    def __init__(self, msg, duplicates):
+        super(DuplicateFeatureVectorsException, self).__init__(msg)
+        self.duplicates = duplicates
+
+
 def extract_numeric(filename):
     found = NUMERIC_VALUE.findall(filename)
     if len(found) == 0:
@@ -34,23 +69,34 @@ class BaseClassificationDatabasePacker(object):
     version = 0
     extension = 'zip'
 
-    def __init__(self, source=None, destination=None, progress_bar=False):
+    def __init__(self, source=None, destination=None, progress_bar=False, check_duplicates=False):
+        if source is not None and destination is not None:
+            raise ValueError("You can't sent source and destination at a same time.")
+
         self.source = source
+
         self.meta_data = {
             'packed': datetime.now().strftime('%c'),
             'sdk_version': rebotics_sdk.__version__,
             'packer_version': self.version
         }
+        self.zip_io = None
+
+        if self.source is not None:
+            self.read_meta()
+
+        self.check_duplicates = check_duplicates
         self.images = []
 
-        if destination is None:
+        if hasattr(destination, 'write') or hasattr(destination, 'read'):
+            self.destination = destination  # file like object
+        elif destination is None:
             self.destination = BytesIO()
         elif isinstance(destination, str) or isinstance(destination, pathlib.PurePath):
             # in python 2 it doesn't accept pathlib PurePath, but only os.path or str
             self.destination = str(destination)
-        else:
-            # don't change if we don't know the type
-            self.destination = destination
+            if not self.destination.endswith(self.extension):
+                self.destination = "{}.{}".format(self.destination, self.extension)
 
         self.progress_bar = progress_bar
 
@@ -77,12 +123,32 @@ class BaseClassificationDatabasePacker(object):
             except RuntimeError:
                 pass
 
+    def __enter__(self):
+        if self.zip_io is not None:
+            return self
+
+        # check for file to be not closed
+
+        if self.source is not None:
+            self.zip_io = self.zipfile(self.source, mode='r')
+        elif self.destination is not None:
+            self.zip_io = self.zipfile(self.destination, mode='w')
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.zip_io.close()
+        self.zip_io = None
+
     def pack(self, *args, **kwargs):
         raise NotImplementedError()
 
     def unpack(self):
         raise NotImplementedError()
 
+    def read_meta(self):
+        with self as packer:
+            self.meta_data = json.load(packer.zip_io.open('meta.json'))
+
 
 class ZipDatabasePacker(BaseClassificationDatabasePacker):
     """
@@ -121,12 +187,9 @@ class ZipDatabasePacker(BaseClassificationDatabasePacker):
             labels = self.read_lines(zip_io.read('labels.txt').decode('utf-8'))
             features = self.read_lines(zip_io.read('features.txt').decode('utf-8'))
 
-            for label, feature in zip(
-                labels,
-                features,
-            ):
+            for index, (label, feature) in enumerate(zip(labels, features)):
                 yield ClassificationEntry(
-                    label, feature, None, None
+                    label, feature, None, None, index
                 )
 
 
@@ -186,6 +249,33 @@ class ClassificationDatabasePacker(BaseClassificationDatabasePacker):
             'images': [str(img.filename) for img in self.images]
         })
 
+        if not self.check_duplicates:
+            return
+
+        # checking for the uniqueness of the FV
+        features_map = defaultdict(list)
+        for i, fv in tqdm.tqdm(enumerate(features), total=features_count, disable=not self.progress_bar, leave=False):
+            md_ = md5(fv.encode('utf-8'))
+            features_map[md_.hexdigest()].append(i)
+        duplicates = []
+        for same_fv_ids in features_map.values():
+            if len(same_fv_ids) > 1:
+                duplicates.append([
+                    ClassificationEntry(
+                        labels[id_],
+                        features[id_],
+                        None,
+                        self.images[id_].filename,
+                        id_
+                    ) for id_ in same_fv_ids
+                ])
+
+        if duplicates:
+            raise DuplicateFeatureVectorsException(
+                "Detected {} duplicate groups.".format(len(duplicates)),
+                duplicates=duplicates
+            )
+
     def pack(self, labels_path, features_path, images_folder):
         self.extract_meta_data(labels_path, features_path, images_folder)
 
@@ -198,7 +288,7 @@ class ClassificationDatabasePacker(BaseClassificationDatabasePacker):
                 for img in self.images:
                     zip_io.write(img.filepath, str(pathlib.Path('images') / img.filename))
             else:
-                for img in tqdm.tqdm(self.images):
+                for img in tqdm.tqdm(self.images, leave=False):
                     zip_io.write(img.filepath, str(pathlib.Path('images') / img.filename))
 
         return self.destination
@@ -213,17 +303,60 @@ class ClassificationDatabasePacker(BaseClassificationDatabasePacker):
             raise ClassificationDatabaseException("Can not unpack with empty source file")
 
         with self.zipfile(self.source, mode='r') as zip_io:
-            self.meta_data = json.load(zip_io.open('meta.json'))
-
             labels = self.read_lines(zip_io.read('labels.txt').decode('utf-8'))
             features = self.read_lines(zip_io.read('features.txt').decode('utf-8'))
 
-            for label, feature, image_name in zip(
+            for index, (label, feature, image_name) in enumerate(zip(
                 labels,
                 features,
                 self.meta_data['images']
-            ):
+            )):
                 image = zip_io.read('images/{}'.format(image_name))
                 yield ClassificationEntry(
-                    label, feature, image, image_name
+                    label, feature, image, image_name, index
                 )
+
+
+class VirtualClassificationDatabasePacker(BaseClassificationDatabasePacker):
+    extension = 'vrcdb'
+    version = 2
+    batch_size = 10000  # smaller json files for the batches, less memory used to unmarshal it
+
+    def __init__(self, *args, **kwargs):
+        super(VirtualClassificationDatabasePacker, self).__init__(*args, **kwargs)
+
+    def unpack(self):
+        if self.source is None:
+            raise ClassificationDatabaseException("Can not unpack with empty source file")
+
+        with self as packer:
+            for batch in self.meta_data['batches']:
+                for entry in json.load(packer.zip_io.open(batch)):
+                    yield VirtualClassificationEntry.from_dict(entry)
+
+    def pack(self, entries):
+        """
+
+        :param list[VirtualClassificationEntry] entries: an iterable that can be passed
+        :return:
+        """
+        with self as packer:
+
+            for batch_number, chunk in enumerate(more_itertools.ichunked(enumerate(entries, start=1), self.batch_size)):
+                data = []
+
+                for i, entry in chunk:
+                    entry.index = i
+                    data.append(entry.to_dict())
+
+                batch_filename = 'batches/batch_{}.json'.format(batch_number)
+                packer.zip_io.writestr(batch_filename, json.dumps(data))
+
+                batches = self.meta_data.get('batches', [])
+                batches.append(batch_filename)
+                self.meta_data['batches'] = batches
+
+            self.meta_data['count'] = i
+            packer.zip_io.writestr('meta.json', json.dumps(self.meta_data))
+
+        return self.destination
