@@ -4,7 +4,7 @@ from collections import namedtuple
 
 import click
 
-from rebotics_sdk.cli.common import shell, roles, configure, run
+from rebotics_sdk.cli.common import shell, roles, configure, run, set_token
 from rebotics_sdk.cli.utils import read_saved_role, process_role, ReboticsCLIContext, app_dir, pass_rebotics_context, \
     GroupWithOptionalArgument
 from rebotics_sdk.providers import ProviderHTTPServiceException
@@ -138,3 +138,4 @@ api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
 api.add_command(run, 'run')
+api.add_command(set_token, 'set_token')
