@@ -89,7 +89,7 @@ class ReboticsCLIContext(object):
 
         if provider is None:
             raise click.ClickException('Role {0} is not configured.\n'
-                                       'Run: \n<app> -r {0} configure'.format(self.role))
+                                       'Run: \n<command> -r {0} configure'.format(self.role))
         return provider
 
     def update_configuration(self, **configuration):
@@ -103,6 +103,10 @@ class ReboticsCLIContext(object):
         else:
             format_full_table(items, max_column_length=max_column_length, keys_to_skip=keys_to_skip)
 
+    def verbose_log(self, message):
+        if self.verbose:
+            click.echo(message)
+
 
 pass_rebotics_context = click.make_pass_decorator(ReboticsCLIContext, True)
 
