@@ -54,11 +54,16 @@ class AdminProvider(ReboticsBaseProvider):
         self.headers['x-retailer-secret-key'] = retailer_secret_key
 
     @remote_service('/api/token-auth/')
-    def token_auth(self, username, password, **kwargs):
-        json_data = self.session.post(data={
+    def token_auth(self, username, password, verification_code=None, **kwargs):
+        payload = {
             'username': username,
-            'password': password
-        })
+            'password': password,
+        }
+
+        if verification_code is not None:
+            payload['verification_code'] = verification_code
+
+        json_data = self.session.post(data=payload)
         self.set_token(json_data['token'])
         return json_data
 
@@ -126,7 +131,7 @@ class AdminProvider(ReboticsBaseProvider):
 
     @remote_service('/api/classification_data/rcdb/{id}/')
     def rcdb_get(self, id_, **kwargs):
-        return self.session.patch(id=id_)
+        return self.session.get(id=id_)
 
     @property
     def rcdb(self):
@@ -137,3 +142,7 @@ class AdminProvider(ReboticsBaseProvider):
         """
         from ..advanced.flows import ReboticsClassificationDatabase
         return ReboticsClassificationDatabase(provider=self)
+
+    @remote_service('/core/core-callback/{id}/')
+    def get_core_callback_data(self, pk):
+        return self.session.get(id=pk)
