@@ -253,6 +253,7 @@ class RetailerProvider(ReboticsBaseProvider):
         set_param(params, 'status', status)
         set_param(params, 'aisle', aisle)
         set_param(params, 'section', section)
+        params.update(kwargs)
 
         data = self.session.get(params=params)
         return PageResult(data)
@@ -432,3 +433,52 @@ class RetailerProvider(ReboticsBaseProvider):
     @remote_service('/api/v1/rebotics/rpc/export-feature-vector/')
     def feature_vectors_export(self):
         return FeatureVectorFlow(self, self.session)
+
+    @remote_service('/api/v1/realograms/scan-batch/bin/')
+    def create_scan_batch_bin(self, items):
+        # TODO: move to pydantic
+        required_keys = {
+            "imageName": str,
+            "timestamp": str,
+            "sensorName": str,
+            "clubId": str,
+            "annotationVersion": int,
+            "locationLabels": {
+                "area": (str, False),
+                "aisle": (str, False),
+                "section": (str, False),
+            },
+            "pose": {
+                "x": float,
+                "y": float,
+                "theta": (float, False)
+            },
+            "locationId": (str, False),
+            "href": str
+        }
+
+        def _validate_field(obj, key, conf):
+            value = obj.get(key)
+            if isinstance(value, dict):
+                return _validate_field(value, key, conf[key])
+
+            required_ = True
+            if isinstance(conf, tuple):
+                type_, required_ = conf
+            else:
+                type_ = conf
+
+            if value is None:
+                if required_:
+                    raise ValueError("Field {} is required".format(key))
+            else:
+                if isinstance(value, type_):
+                    raise ValueError("Field {} does not match required type".format(key, str(type_)))
+
+        for entry in items:
+            if not isinstance(entry, dict):
+                raise ValueError("Entry should be dict")
+            for expected_key, conf in required_keys.items():
+                _validate_field(entry, expected_key, conf)
+
+        return self.session.post(json=items)
