@@ -43,8 +43,7 @@ def test_processing_action_group_invoke():
 
 def test_configure_command():
     runner = CliRunner()
-    from rebotics_sdk.cli.retailer import api
-    result = runner.invoke(api, ['-r', 'local', 'configure'])
+    # result = runner.invoke(api, ['-r', 'local', 'configure'])
 
 
 def test_camera_settings_add_roi():
